<?php

/**
 * @author Kiritateru, DharmaMedia.
 * @package e-Boekhouden.nl
 * @copyright 2011
 */

defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.model');
jimport('joomla.application.component.helper');
jimport('joomla.error.error');

include_once (implode(DS, array(JPATH_COMPONENT, 'models', 'products.php')));
include_once (implode(DS, array(JPATH_COMPONENT, 'models', 'orders.php')));

class eBoekhoudenModelExOrder extends JModel
{
    var $_id;
    var $_products = array();
    var $_customer = array();
    var $_mutations = array();
    var $_db;

    var $_data;
    var $_params;

    function __construct(&$db)
    {
        $this->_db = $db;
        $this->_params = JComponentHelper::getParams('com_eboekhouden');

        return parent::__construct();
    }
    function buildExportOrder()
    {
        $this->getData();

        if (!$this->checkValidData()) {
            return false;
        }
        return $this->buildCustomer()->buildProducts();
    }
    function checkValidData()
    {
        $errors = false;

        if (!$this->_data || !$this->_params) {
            JError::raiseWarning('003', JText::sprintf('COM_EBOEKHOUDEN_ERROR_ORDER_NO_DATA',
                sprintf('%08d', $this->_data->virtuemart_order_item_id)));
            $errors = true;
        }
        if ($this->_data->order_shipment && !$this->params->get('shipping_number')) {
            JError::raiseWarning('003', JText::sprintf('COM_EBOEKHOUDEN_ERROR_ORDER_NO_SHIPPING',
                sprintf('%08d', $this->_data->virtuemart_order_item_id)));
            $errors = true;
        }
        if ($this->_data->order_discount && !$this->params->get('discount_number')) {
            JError::raiseWarning('003', JText::sprintf('COM_EBOEKHOUDEN_ERROR_ORDER_NO_DISCOUNT',
                sprintf('%08d', $this->_data->virtuemart_order_item_id)));
            $errors = true;
        }

        // Anything else?
        
        if ($errors) {
            return false;
        }
        return true;
    }
    function getExportOrder($type = 'array')
    {
        switch (strtolower($type)) {
            case 'array':
                return array('ACTION' => $this->_data->mutation_nr ? 'ALTER_MUTATIE' :
                    'ADD_MUTATIE', 'MUTATIE' => array('NAW' => $this->_customer, 'SOORT' => 2,
                    'REKENING' => 1300, 'INEX' => 'EX', 'FACTUUR' => sprintf('%08d', $this->_data->
                    order_id), 'BETALINGSTERMIJN' => 30, 'DATUM' => date('d-m-Y', strtotime($this->_data->
                    cdate)), 'MUTATIEREGELS' => $this->_mutations));
                break;
        }
    }

    protected function buildCustomer()
    {
        /**
         * Customer Format
         * <NAW>
         *  <BEDRIJF>Yvonne van der A-Mulder</BEDRIJF>
         *  <ADRES>Hondsdraflaan 14</ADRES>
         *  <POSTCODE>2121RA</POSTCODE>
         *  <PLAATS>Bennebroek</PLAATS>
         *  <TELEFOON>023-5849879</TELEFOON>
         *  <EMAIL>info@email.nl</EMAIL>
         * </NAW>
         */

        $this->_customer = array('BEDRIJF' => implode(' ', array($this->_data->
            first_name, $this->_data->middle_name, $this->_data->last_name)), 'ADRES' =>
            implode(' ', array($this->_data->address_1, $this->_data->address_2)),
            'POSTCODE' => $this->_data->zip, 'PLAATS' => $this->_data->city, 'TELEFOON' => $this->
            _data->phone_1, 'EMAIL' => $this->_data->email);

        return $this;
    }
    protected function buildProducts()
    {
        // Somewhere this get's called twice, cba to find it...
        if (!empty($this->_mutations)) {
            return $this;
        }
        if(JRequest::getVar('conf_debug') && JRequest::getVar('debug_id') == 7) {
            var_dump($this->_products);die();
        }
        foreach ($this->_products as $i) {
            $d = $i->getTaxes($this->_data->order_id); // Setup taxes info
            $this->_mutations[] = array('MUTATIEREGEL' => array('BEDRAGINCL' => round($d->
                incl * $d->quantity, 2), 'BEDRAGEXCL' => round($d->excl * $d->quantity, 2),
                'BTWBEDRAG' => round($d->tax_amount * $d->quantity, 2), 'TEGENREKENING' => $i->
                getLargeNumber(), 'BTWPERC' => $d->tax_code));
        }

        // Add shipping mutation and order discount
        $q = eBoekhoudenHelperExternal::getExternalQuery('order.shipping', '',
                'WHERE a.virtuemart_order_id=' . $this->_data->virtuemart_order_id);
        $this->_db->setQuery($q);
        $this->_db->query();
        
        
        $shipping = $this->_db->loadObject();
        $country = $shipping->country_3_code;
        
        $taxArray = eBoekhoudenHelperExternal::getTaxes(floatval($this->_data->
            order_shipment_tax) + floatval($this->_data->order_shipment), floatval($this->
            _data->order_shipment), $country);

        $this->_mutations[] = array('MUTATIEREGEL' => array('BEDRAGINCL' => round($taxArray['inclusive'],
            2), 'BEDRAGEXCL' => round($taxArray['exclusive'], 2), 'BTWBEDRAG' => round($taxArray['tax_amount'],
            2), 'TEGENREKENING' => $this->params->get('shipping_number'), 'BTWPERC' => $taxArray['code']));
        
        if ($this->_data->order_discount != '0.00' || $this->_data->coupon_discount !=
            '0.00') {
                
            $this->_data->coupon_discount = $this->_data->coupon_discount * -1;
            $this->_data->order_discount = $this->_data->order_discount * -1;
            
            $amount = floatval($this->_data->coupon_discount) + floatval($this->_data->
                order_discount);

            $this->_mutations[] = array('MUTATIEREGEL' => array('BEDRAGINCL' => (string )
                round($amount, 2), 'BEDRAGEXCL' => (string )round($amount, 2), 'TEGENREKENING' =>
                $this->params->get('discount_number'), 'BTWPERC' => eBoekhoudenHelperExternal::
                $taxCodes['NONE']));
        }

        return $this;
    }
    protected function getData()
    {
        if (empty($this->_data)) {
            $q = eBoekhoudenHelperExternal::getQuery('order', '*, b.created_on AS cdate', 'WHERE a.id=' . $this->
                _id);

            $this->_db->setQuery($q);
            $this->_db->query();
            $this->_data = $this->_db->loadObject();
            
            $q = eBoekhoudenHelperExternal::getExternalQuery('order.item', '',
                'WHERE virtuemart_order_id=' . $this->_data->order_id);

            $this->_db->setQuery($q);
            $this->_db->query();
            $tmp = $this->_db->loadObjectList();
            if(JRequest::getVar('conf_debug') && JRequest::getVar('debug_id') == 8) {
                var_dump($q); echo '<br />'; var_dump($tmp); echo '<br />';
            }
            foreach ($tmp as $i) {
                if(isset($i->product_id)) {
                    $this->addProduct($i->product_id);
                } else {
                    $this->addProduct($i->virtuemart_product_id);
                }
                
            }
        }
        return $this;
    }
    function addProduct($product_id)
    {
        $tmp = new eBoekhoudenModelProducts();
        $tmp->getData('product_id', $product_id);
        $this->_products[] = $tmp;
    }
    function __get($key)
    {
        if (!is_array($key)) {
            $key = "_$key";
            return $this->$key;
        }

        foreach ($key as $k => $v) {
            switch ($k) {
                case 'product':
                    return $this->getProduct($v);
                    break;
                case 'products':
                    return $this->_products;
                    break;
                case 'customer':
                    return $this->_dataustomer;
                    break;
            }
        }
    }
    function __set($key, $value)
    {
        $key = "_$key";
        $this->$key = $value;
    }
    function getProduct($id)
    {
        foreach ($this->_products as $i) {
            //            if (!$i instanceof eBoekhoudenModelOrders) {
            //                $tmp = new eBoekhoudenModelProducts();
            //                $tmp->setId($i);
            //                $i = $tmp;
            //            }
            if ($i->product_id == $id) {
                return $i;
            }
        }
    }
}
